//
// Example program to illustrate how to read random data from
// the Alea I in a C++ application.
//
// (c) Copyright 2004-2009 Araneus Information Systems Oy.
// All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include <io.h>
#include <fcntl.h>

#include <windows.h>

#include "alea.h"

// The number of bytes to read at a time.

#define BUFFER_SIZE 1024

static void
fatal(char *format, ...) {
    va_list args;
    va_start(args, format);
    vfprintf(stderr, format, args);
    va_end(args);
    fprintf(stderr, "\n");
    exit(1);
}

static void
usage(void) {
    fatal("usage: randomfile [-b] [length]");
}

int _cdecl
main(int argc, char *argv[]) {
    bool binary = false;
    
    argv++, argc--;
    while (argc && argv[0][0] == '-') {
	if (strcmp(argv[0], "-b") == 0) {
	    binary = true;
	} else {
	    usage();
	}
        argv++, argc--;
    }

    // Number of bytes to read, or -1 if no limit
    int nbytes;

    if (argc == 0)
	nbytes = -1;
    else if (argc == 1)
	nbytes = atoi(argv[0]);
    else 
	usage();
    
    if (binary)
	if (_setmode(1, _O_BINARY) < 0)
	    fatal("could not set binary mode");

    alea_t *alea = alea_open();
    if (! alea)
	fatal("could not open Alea device");
    unsigned char buffer[BUFFER_SIZE];
    unsigned int remaining = nbytes == -1 ? BUFFER_SIZE : nbytes;
    while (remaining > 0) {
	unsigned long nreq =
	    remaining <= BUFFER_SIZE ? remaining : BUFFER_SIZE;
	bool r = alea->read(buffer, nreq);
	if (! r)
	    fatal("error %d reading from Alea device", GetLastError());
	for (unsigned int i = 0; i < nreq; i++) {
	    if (binary)
		putchar(buffer[i]);
	    else
		printf("%02x", buffer[i]);
	}
	if (nbytes != -1)
	    remaining -= nreq;
    }

    delete alea;
    return 0;
}
